package edu.kit.ipd.descartes.adaptation.model.util;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Before;
import org.junit.Test;

import edu.kit.ipd.descartes.adaptation.AbstractTest;
import edu.kit.ipd.descartes.adaptation.exceptions.OperationNotPerformedException;
import edu.kit.ipd.descartes.mm.adaptation.AdaptationDirection;
import edu.kit.ipd.descartes.mm.adaptationpoints.AdaptationpointsFactory;
import edu.kit.ipd.descartes.mm.adaptationpoints.ModelVariableConfigurationRange;
import edu.kit.ipd.descartes.mm.resourceconfiguration.ActiveResourceSpecification;
import edu.kit.ipd.descartes.mm.resourceconfiguration.NumberOfParallelProcessingUnits;
import edu.kit.ipd.descartes.mm.resourceconfiguration.ProcessingResourceSpecification;
import edu.kit.ipd.descartes.mm.resourceconfiguration.ResourceconfigurationFactory;
import edu.kit.ipd.descartes.mm.resourcelandscape.Container;
import edu.kit.ipd.descartes.mm.resourcelandscape.ResourcelandscapeFactory;
import edu.kit.ipd.descartes.mm.resourcelandscape.RuntimeEnvironment;

public class DmmModelChangerTest extends AbstractTest {

    private static final String PARENT_CONTAINER = "PARENT_CONTAINER";
    private static final String TEST_RUNTIME_ENV = "TEST_RUNTIME_ENV";
    private static final int MIN = 1;
    private static final int MAX = 4;
    private static final int DEFAULT = 2;
    
    RuntimeEnvironment original = null;
    NumberOfParallelProcessingUnits adaptableNumberOfCores = null;
    ModelVariableConfigurationRange point = null;

    @Before
    public void setUp() throws Exception {
        super.setUp();
        original = ResourcelandscapeFactory.eINSTANCE.createRuntimeEnvironment();
        original.setName(TEST_RUNTIME_ENV);
        
        ActiveResourceSpecification spec = ResourceconfigurationFactory.eINSTANCE.createActiveResourceSpecification();
        ProcessingResourceSpecification procSpec = ResourceconfigurationFactory.eINSTANCE.createProcessingResourceSpecification();
        adaptableNumberOfCores = ResourceconfigurationFactory.eINSTANCE.createNumberOfParallelProcessingUnits();
        
        adaptableNumberOfCores.setNumber(DEFAULT);
        procSpec.setNrOfParProcUnits(adaptableNumberOfCores);
        spec.getProcessingResourceSpecifications().add(procSpec);
        
        original.getConfigSpec().add(spec);
        
        point = AdaptationpointsFactory.eINSTANCE.createModelVariableConfigurationRange();

        point.setMinValue(MIN);
        point.setMaxValue(MAX);
        point.setAdaptableEntity(adaptableNumberOfCores);
    }

    @Test(expected=OperationNotPerformedException.class)
    public void testScaleModelVariableException() throws OperationNotPerformedException {
        assertTrue(adaptableNumberOfCores.getNumber() == DEFAULT);
        DmmModelChanger.scaleModelVariable(point, AdaptationDirection.INCREASE);
        DmmModelChanger.scaleModelVariable(point, AdaptationDirection.INCREASE);
        DmmModelChanger.scaleModelVariable(point, AdaptationDirection.INCREASE);
    }
    
    @Test
    public void testScaleModelVariable() throws OperationNotPerformedException {
        
        DmmModelChanger.scaleModelVariable(point, AdaptationDirection.NOT_SET);
        assertTrue(adaptableNumberOfCores.getNumber() == DEFAULT);
        
        DmmModelChanger.scaleModelVariable(point, AdaptationDirection.INCREASE);
        assertTrue(adaptableNumberOfCores.getNumber() == DEFAULT + 1);
        
        DmmModelChanger.scaleModelVariable(point, AdaptationDirection.INCREASE);
        assertTrue(adaptableNumberOfCores.getNumber() == DEFAULT + 2);

        DmmModelChanger.scaleModelVariable(point, AdaptationDirection.DECREASE);
        DmmModelChanger.scaleModelVariable(point, AdaptationDirection.DECREASE);
        DmmModelChanger.scaleModelVariable(point, AdaptationDirection.DECREASE);
        assertTrue(adaptableNumberOfCores.getNumber() == DEFAULT - 1);
        

    }

    @Test
    public void testScaleInRuntimeEnvironmentInstance() {
        fail("Not yet implemented");
    }

    @Test
    public void testScaleOutRuntimeEnvironmentInstance() {
        Container parent = ResourcelandscapeFactory.eINSTANCE.createComputingInfrastructure();
        
        parent.setName(PARENT_CONTAINER);
        parent.getContains().add(original);
        
        RuntimeEnvironment copy = DmmModelChanger.scaleOutRuntimeEnvironmentInstance(original);
        
        assertFalse(original.getId().equals(copy.getId()));
        assertFalse(original.getName().equals(copy.getName()));
        assertEquals(original.getName() + DmmModelChanger.COPY_MARK, copy.getName());
        assertEquals(original.getTemplate(), copy.getTemplate());
        
        assertTrue(parent.getContains().size() == 2);
        
        assertEquals(original, parent.getContains().get(0));
        assertEquals(copy, parent.getContains().get(1));
    }

}
